import {FunctionComponent, ReactElement, ReactNode} from 'react';
import {Reflection, ReflectionCategory, ReflectionGroup} from 'typedoc';

export type Node = {
  id: string;
  name: string;
  url: string;
  skip: boolean;
  hide: boolean;
  children: Node[];
  parent?: Node;
  group?: ReflectionGroup;
  category?: ReflectionCategory;
  reflection?: Reflection;
  summary?: string;
  body?: string;
  executables?: Executables;
  mentions: Map<Node, number>;
  pages: Map<
    string,
    {
      pageComponent: NoPropComponent;
      isSingle: boolean;
      output: 'html' | 'text' | 'markdown';
      html?: string;
    }
  >;
};

export type SortList = (string | RegExp)[];
export type Sorter = (str1: string, str2: string) => number;
export type NodeSorters = {
  group?: Sorter;
  category?: Sorter;
  reflection?: Sorter;
};

type Executables = {
  base?: string;
  html?: string;
  htmlDiffs?: string[];
  less?: string;
  lessDiffs?: string[];
  tsx?: string;
  tsxDiffs?: string[];
};
type NoPropComponent = FunctionComponent<Record<string, never>>;

export function NodeBreadcrumbs(props: {
  node: Node;
}): ReactElement<any, any> | null;
export function NodeChildren(props: {
  node: Node;
}): ReactElement<any, any> | null;
export function NodeName(props: {
  node: Node;
  link?: boolean;
}): ReactElement<any, any> | null;
export function NodeNavigation(props: {
  node: Node;
  json?: boolean;
}): ReactElement<any, any> | null;
export function NodePreamble(props: {
  node: Node;
  asText?: boolean;
}): ReactElement<any, any> | null;
export function NodeSection(props: {
  node: Node;
  summary?: boolean;
}): ReactElement<any, any> | null;
export function NodeSummary(props: {
  node: Node;
  asText?: boolean;
}): ReactElement<any, any> | null;
export function Markdown(props: {
  markdown: string;
  html?: boolean;
  skipCode?: boolean;
}): ReactElement<any, any> | null;
export function Table(props: {
  children: ReactNode;
}): ReactElement<any, any> | null;

export function useBaseUrl(): string;
export function useRootNode(): Node;
export function usePageNode(): Node;
export function useAllNodes(): Node[];
export function useIsSingle(): boolean;

export interface Docs {
  addApiDir(apiDir: string): Docs;
  addApiFile(apiFile: string): Docs;

  addRootMarkdownFile(markdownFile: string): Docs;
  addMarkdownDir(markdownDir: string, executable?: boolean): Docs;

  addJsDir(jsDir: string, destinationDir?: string): Docs;
  addJsFile(jsFile: string, destinationDir?: string): Docs;

  addLessDir(lessDir: string, destinationDir?: string): Docs;
  addLessFile(lessFile: string, destinationDir?: string): Docs;

  addDir(dir: string, destinationDir?: string): Docs;
  addFile(file: string, destinationDir?: string): Docs;
  addFiles(dir: string, suffixes: string[], destinationDir?: string): Docs;

  addStringFile(content: string, destinationFile: string): Docs;
  addConcatFiles(files: string[], destinationFile: string): Docs;

  generateNodes(nodeSorters?: {
    group?: Sorter;
    category?: Sorter;
    reflection?: Sorter;
  }): Docs;

  addPageForNode(
    url: string,
    pageComponent?: NoPropComponent,
    file?: string,
    single?: boolean,
  ): Docs;
  addPageForEachNode(
    url: string,
    pageComponent?: NoPropComponent,
    file?: string,
  ): Docs;

  addTextForNode(
    url: string,
    textComponent?: NoPropComponent,
    file?: string,
    single?: boolean,
  ): Docs;
  addTextForEachNode(
    url: string,
    textComponent?: NoPropComponent,
    file?: string,
  ): Docs;

  addMarkdownForNode(
    url: string,
    markdownComponent?: NoPropComponent,
    file?: string,
    single?: boolean,
  ): Docs;
  addMarkdownForEachNode(
    url: string,
    markdownComponent?: NoPropComponent,
    file?: string,
  ): Docs;

  publish(): void;
}
export function createDocs(
  host: string,
  rootDir: string,
  reUseDir?: boolean,
): Docs;

export function getSkippedChildren(node: Node): Node[];

export function isAncestor(node: Node, nodeAncestor: Node): boolean;

export function isAncestorOrSame(node: Node, nodeAncestor: Node): boolean;

export function getSorter(sortList: SortList): Sorter;

export function forEachDirAndFile(
  dir: string,
  dirCallback: ((dir: string) => void) | null,
  fileCallback?: (file: string) => void,
): void;
