import { ReflectionKind, ContainerReflection, ReflectionCategory, DeclarationReflection, Application, Converter } from 'typedoc';
import { resolve, join, dirname, basename, extname } from 'path';
import { readdirSync, rmSync, existsSync, mkdirSync, readFileSync, writeFileSync, copyFileSync } from 'fs';
import React, { createContext, useContext, useMemo, createElement, Children, Fragment } from 'react';
import ReactMarkdown from 'react-markdown';
import rehypeRaw from 'rehype-raw';
import remarkGfm from 'remark-gfm';
import { source } from 'unist-util-source';
import { applyPatch } from 'diff';
import less from 'less';
import { minify } from 'html-minifier';
import { transformSync, buildSync } from 'esbuild';
import 'prismjs/plugins/diff-highlight/prism-diff-highlight.js';
import Prism from 'prismjs';
import he from 'he';
import loadLanguages from 'prismjs/components/index.js';
import { renderToStaticMarkup } from 'react-dom/server';

const UTF8 = "utf-8";
const exists = existsSync;
const forEachDeepFile = (dir, callback) => forEachDirAndFile(
  dir,
  (dir2) => forEachDeepFile(dir2, callback),
  (file) => callback(file)
);
const forEachDirAndFile = (dir, dirCallback, fileCallback) => readdirSync(dir, { withFileTypes: true }).forEach((entry) => {
  const path = resolve(join(dir, entry.name));
  entry.isDirectory() ? dirCallback?.(path) : fileCallback?.(path);
});
const rmDir = (dir) => rmSync(dir, { recursive: true, force: true });
const makeDir = (dir) => {
  if (!existsSync(dir)) {
    mkdirSync(dir, { recursive: true });
  }
  return dir;
};
const readFile = (file) => readFileSync(file, UTF8);
const makeFile = (file, content) => writeFileSync(join(makeDir(dirname(file)), basename(file)), content, UTF8);
const copyFile = (file, destinationDir) => copyFileSync(file, join(makeDir(destinationDir), basename(file)));

const DIFFS = {
  base: /(?<=\[base\]: # ').*?(?=')/gms,
  html: /(?<=```html)\n.*?(?=```)/gms,
  htmlDiffs: /(?<=```diff-html)\n.*?(?=```)/gms,
  tsx: /(?<=```[jt]sx?)\n.*?(?=```)/gms,
  tsxDiffs: /(?<=```diff-[jt]sx?)\n.*?(?=```)/gms,
  less: /(?<=```less)\n.*?(?=```)/gms,
  lessDiffs: /(?<=```diff-less)\n.*?(?=```)/gms
};
const getExecutablesFromBody = (body) => {
  const executables = {};
  Object.entries(DIFFS).forEach(([key, regex]) => {
    const match = body.match(regex);
    if (match == null) {
      return;
    }
    executables[key] = key.endsWith("Diffs") ? match : match.join("");
  });
  return executables;
};

const defaultSorter = (string1, string2) => string1 > string2 ? 1 : -1;
const createNode = (id, name, parent, extra = {}, fragment = name) => {
  const node = {
    id,
    name,
    url: parent == null ? "/" : getUrl(parent.url, fragment),
    skip: false,
    hide: extra.summary?.startsWith("->") ?? false,
    children: [],
    pages: /* @__PURE__ */ new Map(),
    mentions: /* @__PURE__ */ new Map(),
    ...extra
  };
  if (parent != null) {
    node.parent = parent;
    parent.children.push(node);
  }
  return node;
};
const mountProjectIntoNode = (reflection, node, nodeSorters = {}) => {
  reflection.getReflectionsByKind(ReflectionKind.Module).sort(
    (child1, child2) => (nodeSorters.reflection ?? defaultSorter)(child1.name, child2.name)
  ).forEach((module) => {
    createContainerNode(module, node, nodeSorters);
  });
  return node;
};
const createMarkdownNode = (dir, parent, executable) => {
  const {
    name = basename(dir),
    summary,
    body,
    executables
  } = getMarkdown(join(dir, "index.md"), executable);
  const node = createNode(getId(name), name, parent, {
    summary,
    body,
    executables
  });
  forEachDirAndFile(
    dir,
    (dir2) => createMarkdownNode(dir2, node, executable),
    (file) => {
      if (extname(file) == ".md" && basename(file) != "index.md") {
        const {
          name: name2 = basename(file, ".md"),
          summary: summary2,
          body: body2,
          executables: executables2
        } = getMarkdown(file, executable);
        createNode(getId(name2), name2, node, {
          summary: summary2,
          body: body2,
          executables: executables2
        });
      }
    }
  );
  return node;
};
const getMarkdown = (path, executable = false) => {
  if (!exists(path)) {
    return {};
  }
  const markdown = readFile(path);
  const nameLongBody = markdown.trim().match(/#(.*?)\n(.*)/s);
  if (nameLongBody == null) {
    return {};
  }
  let [, name, longBody] = nameLongBody;
  name = name.trim();
  longBody = longBody.trim();
  const summaries = [];
  const bodies = [];
  let foundSummary = false;
  longBody.split("\n\n").forEach((section) => {
    (foundSummary ? bodies : summaries).push(section);
    if (!foundSummary && !section.startsWith("#")) {
      foundSummary = true;
    }
  });
  const summary = summaries.join("\n\n").trim();
  const body = bodies.join("\n\n").trim();
  if (body.length == 0 || !executable) {
    return { name, summary, body };
  }
  return { name, summary, body, executables: getExecutablesFromBody(body) };
};
const prePublish = (node) => {
  node.skip = node.children.length == 1 && node.children[0].reflection == null;
};
const getSkippedChildren = (node) => node.skip ? getSkippedChildren(node.children[0]) : node.children.filter((child) => !child.hide);
const isAncestor = (node, nodeAncestor) => node.url.startsWith(nodeAncestor.url);
const isAncestorOrSame = (node, nodeAncestor) => isAncestor(node, nodeAncestor) || node == nodeAncestor;
const getSlug = (str) => str.replace(/[^a-zA-Z0-9]+/g, " ").trim().toLowerCase().replace(/\s+/g, "-");
const getUrl = (url, fragment) => url + getSlug(fragment) + "/";
const reactNodeToString = (childrenOrChild) => (typeof childrenOrChild == "string" || typeof childrenOrChild == "number" || typeof childrenOrChild == "boolean" ? childrenOrChild.toString() : Array.isArray(childrenOrChild) ? childrenOrChild.map((child) => reactNodeToString(child)).join(" ") : reactNodeToString(childrenOrChild?.props?.children)).replace(/\s+/gms, " ");
const sanitizeCategoryTitle = (categoryTitle, groupTitle) => groupTitle == "Functions" && (categoryTitle.endsWith(" hooks") || categoryTitle.endsWith(" components")) ? categoryTitle : categoryTitle + " " + groupTitle.toLowerCase();
const createContainerNode = (reflection, parent, nodeSorters = {}) => {
  const reflectionId = getId(reflection.name);
  const node = createNode(`${reflectionId}`, reflection.name, parent, {
    reflection
  });
  if (reflection instanceof ContainerReflection) {
    (reflection.groups ?? []).sort(
      (group1, group2) => (nodeSorters.group ?? defaultSorter)(group1.title, group2.title)
    ).forEach((group) => {
      const groupId = getId(group.title);
      if (group.categories == null) {
        const category = new ReflectionCategory("Other");
        category.children.push(...group.children);
        group.categories = [category];
      }
      const groupNode = createNode(
        `${reflectionId}/${groupId}`,
        group.title,
        node,
        { group }
      );
      group.categories.sort(
        (category1, category2) => (nodeSorters.category ?? defaultSorter)(
          category1.title,
          category2.title
        )
      ).forEach((category) => {
        const categoryId = getId(category.title);
        const categoryNode = createNode(
          `${reflectionId}/${groupId}/${categoryId}`,
          sanitizeCategoryTitle(category.title, group.title),
          groupNode,
          { category },
          category.title
        );
        category.children.sort(
          (child1, child2) => (nodeSorters.reflection ?? defaultSorter)(
            child1.name,
            child2.name
          )
        ).forEach(
          (reflection2) => createContainerNode(reflection2, categoryNode, nodeSorters)
        );
      });
    });
  }
  return node;
};
const ids = /* @__PURE__ */ new Set();
const getId = (name) => {
  const prefix = name.match(/^.|[A-Z]/g)?.join("") ?? "-";
  let id = prefix;
  let count = 1;
  while (ids.has(id)) {
    id = `${prefix}${++count}`;
  }
  ids.add(id);
  return id;
};

const Context$4 = createContext({});
const PageContext = ({
  baseUrl,
  rootNode,
  pageNode = rootNode,
  allNodes,
  isSingle,
  children
}) => /* @__PURE__ */ React.createElement(
  Context$4.Provider,
  {
    value: useMemo(
      () => ({ baseUrl, rootNode, pageNode, allNodes, isSingle }),
      [baseUrl, rootNode, pageNode, allNodes, isSingle]
    )
  },
  children
);
const useBaseUrl = () => useContext(Context$4).baseUrl;
const useRootNode = () => useContext(Context$4).rootNode;
const usePageNode = () => useContext(Context$4).pageNode;
const useAllNodes = () => useContext(Context$4).allNodes;
const useIsSingle = () => useContext(Context$4).isSingle;

const Context$3 = createContext({ inCode: false });
const Code = ({ children }) => /* @__PURE__ */ React.createElement(Context$3.Provider, { value: useMemo(() => ({ inCode: true }), []) }, /* @__PURE__ */ React.createElement("code", { className: "language-ts" }, children));
const useInCode = () => useContext(Context$3).inCode;

const NodeName = ({
  node,
  link = false
}) => {
  const href = (useIsSingle() ? "#" : "") + node.url;
  if (useInCode()) {
    return node.name;
  }
  const label = node.reflection != null ? /* @__PURE__ */ React.createElement("code", null, node.name) : node.name;
  return link ? /* @__PURE__ */ React.createElement("a", { href }, label) : label;
};

const NodeBreadcrumbs = ({
  node
}) => {
  const pageNode = usePageNode();
  return useIsSingle() ? null : isAncestor(pageNode, node) ? /* @__PURE__ */ React.createElement(React.Fragment, null, /* @__PURE__ */ React.createElement("li", null, /* @__PURE__ */ React.createElement(NodeName, { node, link: true })), getSkippedChildren(node).map((node2, key) => /* @__PURE__ */ React.createElement(NodeBreadcrumbs, { node: node2, key }))) : null;
};

const NodeNavigation = ({
  node
}) => {
  const pageNode = usePageNode();
  const isSingle = useIsSingle();
  const children = getSkippedChildren(node);
  const isParent = children.length > 0;
  const isOpen = isSingle || isAncestorOrSame(pageNode, node);
  const classes = [];
  if (!isSingle && node == pageNode) {
    classes.push("current");
  }
  if (isParent) {
    classes.push("parent");
    if (isOpen) {
      classes.push("open");
    }
  }
  const attrs = { id: node.id };
  if (classes.length > 0) {
    attrs.className = classes.join(" ");
  }
  return /* @__PURE__ */ React.createElement("li", { ...attrs }, /* @__PURE__ */ React.createElement("span", null), /* @__PURE__ */ React.createElement(NodeName, { node, link: true }), isOpen && isParent ? /* @__PURE__ */ React.createElement("ul", null, children.map((node2, key) => /* @__PURE__ */ React.createElement(NodeNavigation, { node: node2, key }))) : null);
};

const Heading = ({
  level,
  id,
  children
}) => createElement(`h${Math.min(level, 6)}`, id != null ? { id } : {}, children);

const Table = ({
  className,
  children
}) => /* @__PURE__ */ React.createElement("div", { className: "table" }, /* @__PURE__ */ React.createElement("table", { ...className != null ? { className } : {} }, children));

const TS_OVERRIDE = /```ts override(.*?)```/s;
const extractTsOverride = (commented) => TS_OVERRIDE.exec(commented.comment?.text ?? "")?.[1]?.trim();
const removeTsOverride = (markdown) => markdown.replace(TS_OVERRIDE, "");

const Context$2 = createContext({});
const NodeContext = ({
  node,
  children
}) => /* @__PURE__ */ React.createElement(Context$2.Provider, { value: useMemo(() => ({ node }), [node]) }, children);
const useNode = () => useContext(Context$2).node;

const Context$1 = createContext({ level: 1 });
const Section = ({
  title,
  id,
  dataId,
  children
}) => {
  const { level } = useContext(Context$1);
  return /* @__PURE__ */ React.createElement(Context$1.Provider, { value: useMemo(() => ({ level: level + 1 }), [level]) }, /* @__PURE__ */ React.createElement(
    "section",
    {
      className: `s${level}`,
      ...id != null ? { id } : {},
      ...dataId != null ? { "data-id": dataId } : {}
    },
    /* @__PURE__ */ React.createElement(Heading, { level }, title),
    children
  ));
};
const useSectionLevel = () => useContext(Context$1).level;

const FENCES = "```";
const bumpHeader = (sectionLevel) => ({ level, children }) => /* @__PURE__ */ React.createElement(
  Heading,
  {
    level: sectionLevel + level - 1,
    id: getSlug(reactNodeToString(children) ?? "")
  },
  children
);
const Block = ({ children }) => /* @__PURE__ */ React.createElement("section", null, children);
const skipCodeNode = (node, file) => {
  node.children?.forEach((node2) => {
    if (node2.type == "element" && node2.tagName == "pre") {
      node2.children = [{ type: "text", value: source(node2.position, file) }];
    } else {
      skipCodeNode(node2, file);
    }
  });
};
const markdownLinkers = /* @__PURE__ */ new Set([
  {
    prefix: "(^|\\W)",
    kinds: [],
    hasMarkdown: true,
    suffix: "($|\\W)",
    nodes: /* @__PURE__ */ new Map()
  },
  {
    prefix: "(^|\\W)",
    kinds: [ReflectionKind.TypeAlias, ReflectionKind.Interface],
    suffix: "($|\\W)",
    nodes: /* @__PURE__ */ new Map()
  },
  {
    prefix: "(^|\\W)",
    kinds: [ReflectionKind.Module],
    suffix: "\\smodule|\\s*\\|",
    nodes: /* @__PURE__ */ new Map()
  },
  {
    prefix: "(^|\\W)",
    kinds: [ReflectionKind.Method],
    suffix: "(\\smethod)|(\\(\\))|\\s*\\|",
    nodes: /* @__PURE__ */ new Map()
  },
  {
    prefix: "(^|\\W)",
    kinds: [ReflectionKind.Function],
    suffix: "(\\s(function|component|Component|hook|callback))|(\\(\\))|\\s*\\|",
    nodes: /* @__PURE__ */ new Map()
  }
]);
let builtMarkdownLinkers = false;
const Markdown = ({
  markdown,
  html = false,
  skipCode = false
}) => {
  const sectionLevel = useSectionLevel();
  const node = useNode();
  const allNodes = useAllNodes();
  const isSingle = useIsSingle();
  if (!builtMarkdownLinkers) {
    builtMarkdownLinkers = true;
    markdownLinkers.forEach((parts) => {
      const { prefix, kinds, hasMarkdown = false, suffix, nodes } = parts;
      allNodes.forEach((node2) => {
        if (node2.reflection?.kindOf(kinds) || hasMarkdown && node2.summary != null) {
          const linkableName = node2.name;
          if (linkableName != null) {
            if (nodes.get(linkableName) === void 0) {
              nodes.set(linkableName, node2);
            } else {
              console.log(`Duplicate regex ${prefix}${linkableName}${suffix}`);
              nodes.set(linkableName, null);
            }
          }
        }
      });
      parts.regex = new RegExp(
        `(?<=${prefix})(${[...nodes.keys()].map(
          (name) => name.replace(/[-/\\^$*+?.()|[\]{}]/gms, "\\$&").replace(/\s+/gms, "\\s+")
        ).join("|")})(?=${suffix})`,
        "gms"
      );
    });
  }
  return /* @__PURE__ */ React.createElement(
    ReactMarkdown,
    {
      remarkPlugins: [remarkGfm],
      rehypePlugins: [].concat(html ? [rehypeRaw] : []).concat(skipCode ? [() => skipCodeNode] : []),
      components: {
        h1: bumpHeader(sectionLevel),
        h2: bumpHeader(sectionLevel),
        h3: bumpHeader(sectionLevel),
        h4: bumpHeader(sectionLevel),
        h5: bumpHeader(sectionLevel),
        h6: bumpHeader(sectionLevel),
        blockquote: Block,
        table: Table
      }
    },
    markdown == null ? "" : removeTsOverride(markdown).split(FENCES).map((block, i) => {
      if (i % 2 == 0) {
        markdownLinkers.forEach(({ regex, nodes }) => {
          block = regex == null ? block : block.split(/[[\]]/g).map((prose, p) => {
            if (p % 2 == 0) {
              return prose.replace(regex, (token) => {
                token = token.replace(/\s+/gms, " ");
                const linkNode = nodes.get(token);
                if (linkNode?.reflection != null) {
                  token = `\`${token}\``;
                }
                if (linkNode == null || linkNode == node) {
                  return token;
                }
                if (node.group == null && node.category == null && node.url != "/") {
                  linkNode.mentions.set(
                    node,
                    (linkNode.mentions.get(node) ?? 0) + 1
                  );
                }
                return `[${token}](${isSingle ? "#" : ""}${linkNode.url})`;
              });
            } else {
              return `[${prose}]`;
            }
          }).join("");
        });
      }
      return block;
    }).join(FENCES)
  );
};

const sanitizeTagName = (tagName, plural) => tagName == "see" ? "See also" : tagName == "example" ? "Example" + (plural ? "s" : "") : tagName[0].toUpperCase() + tagName.substr(1);
const Comment = ({
  comment,
  parentComment,
  part = "both"
}) => {
  if (comment == null) {
    return null;
  }
  const parts = [];
  if (part != "body" && comment.shortText != parentComment?.shortText) {
    parts.push(/* @__PURE__ */ React.createElement(Markdown, { markdown: comment.shortText }));
  }
  if (part != "summary" && comment.text != parentComment?.text) {
    parts.push(/* @__PURE__ */ React.createElement(Markdown, { markdown: comment.text }));
    const tags = /* @__PURE__ */ new Map();
    comment.tags?.map(({ tagName, text }) => {
      if (tagName != "category") {
        if (!tags.has(tagName)) {
          tags.set(tagName, []);
        }
        tags.get(tagName).push(text);
      }
    });
    [...tags.keys()].sort().forEach((tagName) => {
      if (tagName != "param" && tagName != "returns") {
        parts.push(
          /* @__PURE__ */ React.createElement(
            Section,
            {
              title: sanitizeTagName(
                tagName,
                (tags.get(tagName)?.length ?? 0) > 1
              ),
              key: tagName
            },
            tags.get(tagName)?.map((text, key) => /* @__PURE__ */ React.createElement(Markdown, { markdown: text, key }))
          )
        );
      }
    });
  }
  return parts;
};

const Pre = ({ children }) => /* @__PURE__ */ React.createElement("pre", null, /* @__PURE__ */ React.createElement(Code, null, children));

const INDENT = 2;
const Context = createContext({ indent: 0 });
const Separated = ({
  prefix = "",
  suffix = "",
  separator = "",
  multiLine = false,
  children
}) => {
  const multiple = Children.count(children) > 1;
  multiLine = multiLine && multiple;
  const { indent } = useContext(Context);
  const nextIndent = useMemo(
    () => ({ indent: indent + (multiLine ? INDENT : 0) }),
    [indent, multiLine]
  );
  return Children.count(children) == 0 ? null : /* @__PURE__ */ React.createElement(React.Fragment, null, prefix, /* @__PURE__ */ React.createElement(Context.Provider, { value: nextIndent }, Children.map(children, (child, key) => /* @__PURE__ */ React.createElement(Fragment, { key }, multiLine ? "\n" + " ".repeat(indent + INDENT) : key > 0 ? " " : null, child, multiple && key < Children.count(children) - 1 || multiLine ? separator : null))), multiLine ? "\n" + " ".repeat(indent) : null, suffix);
};

const TypeParameters = ({
  typeParameters
}) => /* @__PURE__ */ React.createElement(Separated, { prefix: "<", suffix: ">", separator: "," }, typeParameters?.map((parameter) => parameter.name));

const Type = ({
  type,
  brackets = false
}) => {
  if (type == null) {
    return "any";
  }
  const prefix = brackets ? "(" : "";
  const suffix = brackets ? ")" : "";
  switch (type.type) {
    case "array": {
      const arrayType = type;
      return /* @__PURE__ */ React.createElement(React.Fragment, null, /* @__PURE__ */ React.createElement(Type, { type: arrayType.elementType, brackets: true }), "[]");
    }
    case "conditional": {
      const conditionalType = type;
      return /* @__PURE__ */ React.createElement(React.Fragment, null, prefix, /* @__PURE__ */ React.createElement(Type, { type: conditionalType.checkType, brackets: true }), " extends ", /* @__PURE__ */ React.createElement(Type, { type: conditionalType.extendsType }), " ? ", /* @__PURE__ */ React.createElement(Type, { type: conditionalType.trueType }), " : ", /* @__PURE__ */ React.createElement(Type, { type: conditionalType.falseType }), suffix);
    }
    case "indexedAccess": {
      const indexedAccessType = type;
      return /* @__PURE__ */ React.createElement(React.Fragment, null, /* @__PURE__ */ React.createElement(Type, { type: indexedAccessType.objectType }), "[", /* @__PURE__ */ React.createElement(Type, { type: indexedAccessType.indexType }), "]");
    }
    case "inferred": {
      const inferredType = type;
      return /* @__PURE__ */ React.createElement(React.Fragment, null, "infer ", inferredType.name);
    }
    case "intersection": {
      const intersectionType = type;
      return /* @__PURE__ */ React.createElement(React.Fragment, null, prefix, /* @__PURE__ */ React.createElement(Separated, { separator: " &" }, intersectionType.types.map((type2, key) => /* @__PURE__ */ React.createElement(Type, { type: type2, key }))), suffix);
    }
    case "intrinsic": {
      const intrinsicType = type;
      return intrinsicType.name;
    }
    case "literal": {
      const literalType = type;
      const value = literalType.value;
      return typeof value === "bigint" ? value.toString() + "n" : JSON.stringify(value);
    }
    case "mapped": {
      const mappedType = type;
      return /* @__PURE__ */ React.createElement(React.Fragment, null, "{", mappedType.readonlyModifier === "+" ? "readonly " : mappedType.readonlyModifier === "-" ? "-readonly " : "", "[", mappedType.parameter, " in ", /* @__PURE__ */ React.createElement(Type, { type: mappedType.parameterType }), mappedType.nameType == null ? null : /* @__PURE__ */ React.createElement(React.Fragment, null, "as ", /* @__PURE__ */ React.createElement(Type, { type: mappedType.nameType })), "]", mappedType.readonlyModifier === "+" ? "?: " : mappedType.readonlyModifier === "-" ? "-?: " : ": ", /* @__PURE__ */ React.createElement(Type, { type: mappedType.templateType }), "}");
    }
    case "optional": {
      const optionalType = type;
      return /* @__PURE__ */ React.createElement(React.Fragment, null, /* @__PURE__ */ React.createElement(Type, { type: optionalType.elementType }), "?");
    }
    case "predicate": {
      const predicateType = type;
      return /* @__PURE__ */ React.createElement(React.Fragment, null, predicateType.asserts ? "asserts " : null, predicateType.name, predicateType.targetType == null ? null : /* @__PURE__ */ React.createElement(React.Fragment, null, " is ", /* @__PURE__ */ React.createElement(Type, { type: predicateType.targetType })));
    }
    case "query": {
      const queryType = type;
      return /* @__PURE__ */ React.createElement(React.Fragment, null, "typeof ", /* @__PURE__ */ React.createElement(Type, { type: queryType.queryType }));
    }
    case "reference": {
      const referenceType = type;
      return /* @__PURE__ */ React.createElement(React.Fragment, null, referenceType.name, /* @__PURE__ */ React.createElement(Separated, { prefix: "<", suffix: ">", separator: "," }, referenceType.typeArguments?.map((type2, key) => /* @__PURE__ */ React.createElement(Type, { type: type2, key }))));
    }
    case "reflection": {
      const reflectionType = type;
      return reflectionType.declaration.children != null ? /* @__PURE__ */ React.createElement(React.Fragment, null, "{", /* @__PURE__ */ React.createElement(Separated, { separator: ";", multiLine: true }, reflectionType.declaration.children.map((declaration, key) => /* @__PURE__ */ React.createElement(Fragment, { key }, declaration.getSignature != null ? declaration.setSignature != null ? /* @__PURE__ */ React.createElement(React.Fragment, null, declaration.name, ": ", /* @__PURE__ */ React.createElement(Type, { type: declaration.getSignature.type })) : /* @__PURE__ */ React.createElement(React.Fragment, null, "get ", declaration.name, "(): ", /* @__PURE__ */ React.createElement(Type, { type: declaration.getSignature.type })) : declaration.setSignature != null ? /* @__PURE__ */ React.createElement(React.Fragment, null, "set ", declaration.name, "(", declaration.setSignature.parameters?.[0] == null ? null : /* @__PURE__ */ React.createElement(React.Fragment, null, declaration.setSignature.parameters[0].name, ": ", /* @__PURE__ */ React.createElement(
        Type,
        {
          type: declaration.setSignature.parameters[0].type
        }
      )), ")") : /* @__PURE__ */ React.createElement(React.Fragment, null, declaration.name, declaration.flags.isOptional ? "?: " : ": ", declaration.signatures != null ? declaration.signatures.map((signature, key2) => /* @__PURE__ */ React.createElement(
        SignatureTitle,
        {
          signature,
          key: key2,
          doName: false
        }
      )) : /* @__PURE__ */ React.createElement(Type, { type: declaration.type }))))), "}") : reflectionType.declaration.signatures != null && reflectionType.declaration.signatures.length > 0 ? reflectionType.declaration?.signatures.length > 1 ? /* @__PURE__ */ React.createElement(React.Fragment, null, "{", /* @__PURE__ */ React.createElement(Separated, { separator: ";", multiLine: true }, reflectionType.declaration.signatures.map((signature, key) => /* @__PURE__ */ React.createElement(
        SignatureTitle,
        {
          signature,
          key,
          doName: false
        }
      ))), "}") : /* @__PURE__ */ React.createElement(React.Fragment, null, prefix, /* @__PURE__ */ React.createElement(
        SignatureTitle,
        {
          signature: reflectionType.declaration.signatures[0],
          doName: false,
          doArrow: true
        }
      ), suffix) : reflectionType.declaration.indexSignature != null ? /* @__PURE__ */ React.createElement(React.Fragment, null, "{[", reflectionType.declaration.indexSignature.parameters?.map(
        (parameter, key) => /* @__PURE__ */ React.createElement(Fragment, { key }, parameter.name, ": ", /* @__PURE__ */ React.createElement(Type, { type: parameter.type }))
      ), "]: ", /* @__PURE__ */ React.createElement(Type, { type: reflectionType.declaration.indexSignature.type }), "}") : "{}";
    }
    case "rest": {
      const restType = type;
      return /* @__PURE__ */ React.createElement(React.Fragment, null, "...", /* @__PURE__ */ React.createElement(Type, { type: restType.elementType }));
    }
    case "tuple": {
      const tupleType = type;
      return /* @__PURE__ */ React.createElement(React.Fragment, null, "[", /* @__PURE__ */ React.createElement(Separated, { separator: "," }, tupleType.elements.map((element, key) => /* @__PURE__ */ React.createElement(Type, { type: element, key }))), "]");
    }
    case "template-literal": {
      const templateLiteralType = type;
      return /* @__PURE__ */ React.createElement(React.Fragment, null, "`", templateLiteralType.head, templateLiteralType.tail.map((tail, key) => /* @__PURE__ */ React.createElement(Fragment, { key }, "${", /* @__PURE__ */ React.createElement(Type, { type: tail[0] }), "}", tail[1])), "`");
    }
    case "typeOperator": {
      const typeOperatorType = type;
      return /* @__PURE__ */ React.createElement(React.Fragment, null, typeOperatorType.operator, " ", /* @__PURE__ */ React.createElement(Type, { type: typeOperatorType.target }));
    }
    case "union": {
      const unionType = type;
      return /* @__PURE__ */ React.createElement(React.Fragment, null, prefix, /* @__PURE__ */ React.createElement(Separated, { separator: " |" }, unionType.types.map((type2, key) => /* @__PURE__ */ React.createElement(Type, { type: type2, key, brackets }))), suffix);
    }
    case "unknown": {
      const unknownType = type;
      return unknownType.name;
    }
    case "named-tuple-member": {
      const namedTupleMemberType = type;
      return /* @__PURE__ */ React.createElement(React.Fragment, null, namedTupleMemberType.name, namedTupleMemberType.isOptional ? "?: " : ": ", /* @__PURE__ */ React.createElement(Type, { type: namedTupleMemberType.element }));
    }
  }
};
const SignatureTitle = ({
  signature,
  doName = true,
  doArrow = false
}) => /* @__PURE__ */ React.createElement(React.Fragment, null, doName ? signature.name : signature.kind == ReflectionKind.ConstructorSignature ? /* @__PURE__ */ React.createElement(React.Fragment, null, signature.flags.isAbstract ? "abstract " : null, "new") : null, /* @__PURE__ */ React.createElement(TypeParameters, { typeParameters: signature.typeParameters }), "(", /* @__PURE__ */ React.createElement(Separated, { separator: "," }, signature.parameters?.map((parameter, key) => /* @__PURE__ */ React.createElement(Fragment, { key }, parameter.flags.isRest ? "..." : null, parameter.name, parameter.flags.isOptional ? "?" : null, parameter.defaultValue ? "?" : null, ": ", /* @__PURE__ */ React.createElement(Type, { type: parameter.type })))), ")", /* @__PURE__ */ React.createElement(React.Fragment, null, doArrow ? " => " : ": ", /* @__PURE__ */ React.createElement(Type, { type: signature.type })));

const DeclarationCode = ({
  declaration
}) => /* @__PURE__ */ React.createElement(Pre, null, extractTsOverride(declaration) ?? /* @__PURE__ */ React.createElement(Type, { type: declaration.type }));

const DeclarationsTable = ({
  declarations
}) => declarations.length == 0 ? null : /* @__PURE__ */ React.createElement(Table, null, /* @__PURE__ */ React.createElement("tr", null, /* @__PURE__ */ React.createElement("th", null), /* @__PURE__ */ React.createElement("th", null, "Type"), /* @__PURE__ */ React.createElement("th", null, "Description")), declarations.map((declaration, key) => /* @__PURE__ */ React.createElement("tr", { key }, /* @__PURE__ */ React.createElement("th", null, /* @__PURE__ */ React.createElement(Code, null, declaration.name, declaration.flags.isOptional ? "?" : "")), /* @__PURE__ */ React.createElement("td", null, /* @__PURE__ */ React.createElement(Code, null, declaration.signatures != null ? declaration.signatures.map((signature, key2) => /* @__PURE__ */ React.createElement(SignatureTitle, { signature, key: key2 })) : /* @__PURE__ */ React.createElement(Type, { type: declaration.type }))), /* @__PURE__ */ React.createElement("td", null, declaration.signatures != null ? declaration.signatures.map((signature, key2) => /* @__PURE__ */ React.createElement(Comment, { comment: signature.comment, part: "summary", key: key2 })) : /* @__PURE__ */ React.createElement(Comment, { comment: declaration.comment })))));

const SignatureCode = ({
  signature
}) => /* @__PURE__ */ React.createElement(Pre, null, extractTsOverride(signature) ?? /* @__PURE__ */ React.createElement(React.Fragment, null, signature.name != "__type" ? signature.name : null, /* @__PURE__ */ React.createElement(TypeParameters, { typeParameters: signature.typeParameters }), "(", /* @__PURE__ */ React.createElement(Separated, { separator: ",", multiLine: true }, signature.parameters?.map((parameter, key) => /* @__PURE__ */ React.createElement(Fragment, { key }, parameter.name, parameter.flags.isOptional ? "?: " : ": ", /* @__PURE__ */ React.createElement(Type, { type: parameter.type })))), "): ", /* @__PURE__ */ React.createElement(Type, { type: signature.type })));

const SignatureTable = ({
  signature
}) => {
  const noParameters = (signature.parameters?.length ?? 0) == 0;
  const returnsVoid = signature.type?.type == "intrinsic" && signature.type.name == "void";
  return noParameters && returnsVoid ? null : /* @__PURE__ */ React.createElement(Table, null, noParameters ? null : /* @__PURE__ */ React.createElement(React.Fragment, null, /* @__PURE__ */ React.createElement("tr", null, /* @__PURE__ */ React.createElement("th", null), /* @__PURE__ */ React.createElement("th", null, "Type"), /* @__PURE__ */ React.createElement("th", null, "Description")), signature.parameters?.map((parameter, key) => /* @__PURE__ */ React.createElement("tr", { key }, /* @__PURE__ */ React.createElement("th", null, /* @__PURE__ */ React.createElement(Code, null, parameter.name, parameter.flags.isOptional ? "?" : "")), /* @__PURE__ */ React.createElement("td", null, /* @__PURE__ */ React.createElement(Code, null, /* @__PURE__ */ React.createElement(Type, { type: parameter.type }))), /* @__PURE__ */ React.createElement("td", null, /* @__PURE__ */ React.createElement(Comment, { comment: parameter.comment }))))), /* @__PURE__ */ React.createElement("tr", null, /* @__PURE__ */ React.createElement("th", { className: "right" }, "returns"), /* @__PURE__ */ React.createElement("td", null, /* @__PURE__ */ React.createElement(Code, null, /* @__PURE__ */ React.createElement(Type, { type: signature.type }))), /* @__PURE__ */ React.createElement("td", null, /* @__PURE__ */ React.createElement("p", null, returnsVoid ? "This has no return value." : /* @__PURE__ */ React.createElement(Markdown, { markdown: signature.comment?.returns ?? "" })))));
};

const Signature = ({
  parentComment,
  signature,
  summary
}) => /* @__PURE__ */ React.createElement(React.Fragment, null, signature.comment == null ? null : /* @__PURE__ */ React.createElement(
  Comment,
  {
    comment: signature.comment,
    parentComment,
    part: "summary"
  }
), summary ? null : /* @__PURE__ */ React.createElement(React.Fragment, null, /* @__PURE__ */ React.createElement(SignatureCode, { signature }), /* @__PURE__ */ React.createElement(SignatureTable, { signature }), /* @__PURE__ */ React.createElement(
  Comment,
  {
    comment: signature.comment,
    parentComment,
    part: "body"
  }
)));

const Declaration = ({
  declaration,
  summary = false
}) => {
  const parts = [];
  switch (declaration.kind) {
    case ReflectionKind.Function:
    case ReflectionKind.Method: {
      declaration.signatures?.forEach(
        (signature, key) => parts.push(
          /* @__PURE__ */ React.createElement(Signature, { signature, summary, key })
        )
      );
      break;
    }
    case ReflectionKind.TypeAlias: {
      if (declaration.type?.type == "reflection") {
        const reflectionType = declaration.type;
        const reflectionComment = declaration.reflectionComment;
        if (reflectionType.declaration?.signatures != null) {
          parts.push(/* @__PURE__ */ React.createElement(Comment, { comment: reflectionComment, part: "summary" }));
          reflectionType.declaration.signatures?.forEach(
            (signature, s) => parts.push(
              /* @__PURE__ */ React.createElement(React.Fragment, null, s === 0 || summary ? null : /* @__PURE__ */ React.createElement("hr", null), /* @__PURE__ */ React.createElement(
                Signature,
                {
                  signature,
                  summary,
                  parentComment: reflectionComment
                }
              ))
            )
          );
          if (!summary) {
            parts.push(/* @__PURE__ */ React.createElement(Comment, { comment: reflectionComment, part: "body" }));
          }
        } else {
          parts.push(
            /* @__PURE__ */ React.createElement(
              Comment,
              {
                comment: reflectionType.declaration.comment,
                part: "summary"
              }
            )
          );
          if (!summary) {
            parts.push(
              /* @__PURE__ */ React.createElement(DeclarationCode, { declaration }),
              /* @__PURE__ */ React.createElement(
                DeclarationsTable,
                {
                  declarations: reflectionType.declaration.children ?? []
                }
              ),
              /* @__PURE__ */ React.createElement(
                Comment,
                {
                  comment: reflectionType.declaration.comment,
                  part: "body"
                }
              )
            );
          }
        }
      } else {
        if (!summary) {
          parts.push(/* @__PURE__ */ React.createElement(DeclarationCode, { declaration }));
        }
      }
    }
  }
  if (!summary) {
    parts.push(/* @__PURE__ */ React.createElement(Comment, { comment: declaration.comment, part: "body" }));
  }
  return parts;
};

const { render } = less;
const SCRIPTS_REGEX = /<script.*?<\/script>/gms;
const PURE_REGEX = /\/\* @__PURE__ \*\/ /gms;
const RENDER_REGEX = /ReactDOM\s*\.\s*render/gms;
const RENDER = "__ReactDOM_render";
const CREATE_ELEMENT_REGEX = /React\s*\.\s*createElement/gms;
const CREATE_ELEMENT = "__React_createElement";
const applyDiff = (base, diffs, node) => {
  diffs.forEach((diff) => {
    while (diff.startsWith("\n")) {
      diff = diff.substr(1);
    }
    diff.trimEnd();
    if (diff != "") {
      const next = applyPatch(base, "@@ -0 +0 @@\n" + diff, {
        compareLine: (_1, line, _2, patchContent) => line?.trim() === patchContent?.trim()
      });
      if (next === false || next === base) {
        throw `Could not apply patch in ${node.name}:
BASE:
---${base}---
DIFF:
---${diff}---`;
      }
      base = next;
    }
  });
  return base;
};
const getResolvedExecutables = (node, allNodes) => {
  const {
    base,
    html = "",
    htmlDiffs = [],
    less: less2 = "",
    lessDiffs = [],
    tsx = "",
    tsxDiffs = []
  } = node?.executables ?? {};
  if (base == null) {
    return { html, less: less2, tsx };
  }
  const nodeBase = allNodes.find((node2) => node2.name == base);
  if (nodeBase == null) {
    throw `Base ${base} not present in ${node.name}`;
  }
  const {
    html: htmlBase,
    less: lessBase,
    tsx: tsxBase
  } = getResolvedExecutables(nodeBase, allNodes);
  const executables = {
    html: applyDiff(htmlBase, htmlDiffs, node) + html,
    less: applyDiff(lessBase, lessDiffs, node) + less2,
    tsx: applyDiff(tsxBase, tsxDiffs, node) + tsx
  };
  node.executables = executables;
  return executables;
};
const Executables = ({ node }) => {
  const { html, less: less2, tsx } = getResolvedExecutables(node, useAllNodes());
  if (html == "" && less2 == "" && tsx == "") {
    return null;
  }
  let css = "";
  render(less2, { syncImport: true, compress: true }, (error, result) => {
    if (error != null) {
      throw error;
    }
    if (result != null) {
      css = result.css;
    }
  });
  let js = transformSync(tsx, { loader: "tsx" }).code;
  if (RENDER_REGEX.test(js)) {
    js = js.replace(RENDER_REGEX, RENDER);
    js = `const {render: ${RENDER}} = ReactDOM;${js}`;
  }
  if (CREATE_ELEMENT_REGEX.test(js)) {
    js = js.replace(CREATE_ELEMENT_REGEX, CREATE_ELEMENT);
    js = `const {createElement: ${CREATE_ELEMENT}} = React;${js}`;
  }
  js = transformSync(js, { minify: true, format: "iife" }).code.replace(PURE_REGEX, "").trim();
  const doc = minify(
    `
      <html>
        <head>
          ${html.match(SCRIPTS_REGEX)?.join("")?.trim() ?? ""}
          <style>${css}</style>
        </head>
        <body>
          ${html?.replace(SCRIPTS_REGEX, "")?.trim() ?? ""}
        </body>
        <script>${js}<\/script>
      </html>
    `,
    {
      collapseWhitespace: true,
      minifyCSS: true,
      minifyJS: true
    }
  );
  return /* @__PURE__ */ React.createElement("iframe", { srcDoc: doc });
};

const getReflectionParent = (node) => node.parent && node.parent?.reflection == null ? getReflectionParent(node.parent) : node.parent;
const NodeAutoPreamble = ({
  node,
  asText = false
}) => {
  if (node.reflection == null && node.group == null && node.category == null) {
    return "";
  }
  const children = getSkippedChildren(node);
  return asText ? getIntro(node, children, true) + getInContext(node, true) + "." + getCategoryChildren(node, children, true) : /* @__PURE__ */ React.createElement("p", null, getIntro(node, children), getInContext(node), ".", getCategoryChildren(node, children));
};
const getIntro = (node, children, asText = false) => {
  if (node.group != null) {
    if (children.length == 1) {
      return getNodeText(
        children[0],
        asText,
        `There is one ${children[0].reflection?.kindString?.toLowerCase()}, `,
        ","
      );
    }
    return `These are ${getNoun(node)}`;
  }
  if (node.category != null) {
    return `This is ${getNoun(node)}`;
  }
  if (node.reflection != null) {
    return getNodeText(node, asText, "", ` is ${getNoun(node, true)}`);
  }
};
const getInContext = (node, asText = false) => {
  const parent = getReflectionParent(node);
  if (parent == null || parent.reflection?.kindString == null) {
    return "";
  }
  return getNodeText(
    parent,
    asText,
    " within the ",
    ` ${parent.reflection?.kindString?.toLowerCase()}`
  );
};
const getCategoryChildren = (node, children, asText = false) => {
  if (node.category == null) {
    return "";
  }
  return children.length == 1 ? getNodeText(
    children[0],
    asText,
    ` There is only one ${children[0].reflection?.kindString?.toLowerCase()}, `,
    "."
  ) : getMultiChild(node, children, asText);
};
const getMultiChild = (node, children, asText = false) => {
  const listPrefix = ` There are ${children.length == 2 ? "only two " : children.length == 3 ? "only three " : children.length} ${node.name.toLowerCase()}`;
  return asText ? `${listPrefix}${getChildList(children, true)}.` : /* @__PURE__ */ React.createElement(React.Fragment, null, listPrefix, getChildList(children), ".");
};
const getChildList = (children, asText = false) => {
  return children.length > 3 ? " in total" : asText ? children.reduce(
    (str, child, key) => str + (key == 0 || children.length > 2 ? ", " : "") + (key == children.length - 1 ? " and " : "") + getNodeText(child, true),
    ""
  ) : children.map((child, key) => /* @__PURE__ */ React.createElement(Fragment, { key }, key == 0 || children.length > 2 ? ", " : "", key == children.length - 1 ? " and " : "", getNodeText(child)));
};
const getNodeText = (node, asText = false, prefix = "", suffix = "") => asText ? `${prefix}${node.name}${suffix}` : /* @__PURE__ */ React.createElement(React.Fragment, null, prefix, /* @__PURE__ */ React.createElement(NodeName, { node, link: true }), suffix);
const getNoun = (node, indefinite = false) => {
  if (node.group != null) {
    return `${indefinite ? "some" : "the"} ${node.group.title.toLowerCase()}`;
  }
  if (node.category != null) {
    return `${indefinite ? "a" : "the"} collection of ${node.name.toLowerCase()}`;
  }
  if (node.reflection != null) {
    if (node.reflection.isProject()) {
      return `${indefinite ? "a" : "the"} project`;
    }
    return `${indefinite ? "AEIO".indexOf((node.reflection.kindString ?? "")[0]) != -1 ? "an" : "a" : "the"} ${node.reflection.kindString?.toLowerCase()}`;
  }
  return "";
};

const NodeSummary = ({
  node,
  asText = false
}) => {
  if (node.summary != null) {
    return asText ? normalize(node.summary) : /* @__PURE__ */ React.createElement(Markdown, { markdown: node.summary });
  }
  if (node.reflection instanceof DeclarationReflection) {
    return asText ? normalize(
      // ugh
      node.reflection.comment?.shortText ?? node.reflection?.signatures?.[0]?.comment?.shortText ?? node.reflection?.reflectionComment?.signatures?.[0]?.comment?.shortText ?? node.reflection?.type?.declaration?.signatures?.[0]?.comment?.shortText ?? node.reflection?.type?.declaration?.comment?.shortText
    ) : /* @__PURE__ */ React.createElement(Comment, { comment: node.reflection.comment, part: "summary" });
  }
  if (node.body == null) {
    return asText ? normalize(NodeAutoPreamble({ node, asText })) : /* @__PURE__ */ React.createElement(NodeAutoPreamble, { node });
  }
  return "";
};
const normalize = (str) => (str ?? "").replace(/\s+/g, " ");

const NodeSection = ({
  node,
  summary = false
}) => {
  const children = getSkippedChildren(node);
  return /* @__PURE__ */ React.createElement(NodeContext, { node }, useIsSingle() || !summary ? /* @__PURE__ */ React.createElement(NodeSectionBlock, { node }, children.map((node2, key) => /* @__PURE__ */ React.createElement(NodeSection, { node: node2, key, summary: true }))) : /* @__PURE__ */ React.createElement(NodeSectionBlock, { node, summary: true }, children.length > 0 ? /* @__PURE__ */ React.createElement("ul", null, children.map((node2, key) => /* @__PURE__ */ React.createElement("li", { key }, /* @__PURE__ */ React.createElement(NodeName, { node: node2, link: true })))) : null));
};
const NodeSectionBlock = ({
  node,
  summary = false,
  children
}) => {
  const parts = [/* @__PURE__ */ React.createElement(NodeSummary, { node, key: 0 })];
  if (node.reflection instanceof DeclarationReflection) {
    parts.push(/* @__PURE__ */ React.createElement(Declaration, { declaration: node.reflection, summary }));
  }
  if (!summary) {
    if (node.executables != null) {
      parts.unshift(/* @__PURE__ */ React.createElement(Executables, { node }));
    }
    if (node.body != null) {
      parts.push(/* @__PURE__ */ React.createElement(Markdown, { markdown: node.body }));
    }
  }
  parts.push(children);
  if (!summary && node.reflection != null) {
    parts.push(/* @__PURE__ */ React.createElement(Section, { title: "Mentions" }, node.url));
  }
  return /* @__PURE__ */ React.createElement(
    Section,
    {
      title: /* @__PURE__ */ React.createElement(NodeName, { node, link: summary }),
      id: node.url,
      dataId: node.id
    },
    ...parts
  );
};

const DefaultPage = () => {
  const nodeRoot = useRootNode();
  const pageNode = usePageNode();
  if (pageNode.summary?.startsWith("->")) {
    return /* @__PURE__ */ React.createElement(
      "meta",
      {
        httpEquiv: "refresh",
        content: `0;url=${pageNode.summary.substring(2).trim()}`
      }
    );
  }
  return /* @__PURE__ */ React.createElement("html", null, /* @__PURE__ */ React.createElement("head", null, /* @__PURE__ */ React.createElement("title", null, nodeRoot.name), /* @__PURE__ */ React.createElement(
    "style",
    {
      dangerouslySetInnerHTML: { __html }
    }
  )), /* @__PURE__ */ React.createElement("body", null, /* @__PURE__ */ React.createElement("nav", null, /* @__PURE__ */ React.createElement("ul", null, /* @__PURE__ */ React.createElement(NodeNavigation, { node: nodeRoot }))), /* @__PURE__ */ React.createElement("article", null, /* @__PURE__ */ React.createElement("nav", null, /* @__PURE__ */ React.createElement("ul", null, /* @__PURE__ */ React.createElement(NodeBreadcrumbs, { node: nodeRoot }))), /* @__PURE__ */ React.createElement(NodeSection, { node: pageNode })), /* @__PURE__ */ React.createElement("aside", null, "-")));
};
const __html = `
body {
  display: grid;
  grid-template-columns: minmax(15rem, 1fr) 48rem minmax(15rem, 1fr);
  grid-template-rows: 5rem 1fr 5rem;
  gap: 0 1rem;
  height: 100vh;
  margin: 0;
}

body > header {
  grid-row: 1;
  grid-column: 1 / span 3;
  border-bottom: 1px solid #000;
}

body > nav {
  width: 15rem;
  justify-self: right;
  grid-row: 2;
  grid-column: 1;
  overflow: hidden auto;
  scroll-behavior: smooth;
}

body > article {
  grid-row: 2;
  grid-column: 2;
  overflow: hidden auto;
  scroll-behavior: smooth;
}

body > article > nav li {
  display: inline;
}

body > article > nav li::before {
  content: ' > ';
}

body > aside {
  width: 15rem;
  grid-row: 2;
  grid-column: 3;
  overflow: hidden auto;
}

body > footer {
  grid-row: 3;
  grid-column: 1 / span 3;
  border-top: 1px solid #000;
}

pre {
  padding: 1rem;
}

ul {
  padding: 0;
}

li {
  display: block;
}

li > ul {
  margin-left: 1rem;
}
`.replace(/\s+/g, " ");

const PageWithContext = ({
  baseUrl,
  rootNode,
  pageNode = rootNode,
  allNodes,
  isSingle,
  pageComponent: Page
}) => /* @__PURE__ */ React.createElement(
  PageContext,
  {
    baseUrl,
    rootNode,
    pageNode,
    allNodes,
    isSingle
  },
  /* @__PURE__ */ React.createElement(NodeContext, { node: pageNode }, /* @__PURE__ */ React.createElement(Page, null))
);

const { decode } = he;
const { highlight, hooks, languages } = Prism;
loadLanguages(["jsx", "typescript", "tsx", "diff", "sql"]);
hooks.add("wrap", (env) => {
  env.classes = [env.classes[1]];
});
const TYPES = [ReflectionKind.TypeAlias, ReflectionKind.Interface];
const FUNCTIONS = [ReflectionKind.Function, ReflectionKind.Method];
const codeLinkers = /* @__PURE__ */ new Map([
  ["type", { kinds: TYPES, nodes: /* @__PURE__ */ new Map() }],
  ["function", { kinds: FUNCTIONS, nodes: /* @__PURE__ */ new Map() }],
  ["class-name", { kinds: FUNCTIONS, nodes: /* @__PURE__ */ new Map() }]
]);
let builtCodeLinkers = false;
const removeMagicComments = (code) => code.replace(/ \/\/ !.*?$/gm, "");
const decorate = ({
  html = "",
  isSingle = false,
  output = "html"
}, nodesByUrl, baseUrl) => {
  if (output == "text") {
    return html.trim().replace(/^<\w+>/, "").replace(/<\/\w+>$/, "");
  }
  if (output == "markdown") {
    return minify(html, {
      collapseWhitespace: true,
      ignoreCustomFragments: [/```[\S\s]*?```/gms]
    }).replace(/href="\//gms, `href="${baseUrl}/`).replace(/src="\//gms, `src="${baseUrl}/`).replace(
      /<pre>(```.*?```)<\/pre>/gms,
      (_pre, code) => `

${removeMagicComments(decode(code))}

`
    );
  }
  if (html.startsWith("<html")) {
    html = "<!DOCTYPE html>" + html;
  }
  if (!builtCodeLinkers) {
    builtCodeLinkers = true;
    const types = [...nodesByUrl.values()].filter((node) => node.reflection?.kindOf(TYPES)).map((node) => node.reflection?.name);
    if (types.length > 0) {
      languages.ts.type = new RegExp(`\\b(?:${types.join("|")})\\b`);
      languages.tsx.type = new RegExp(`\\b(?:${types.join("|")})\\b`);
    }
    codeLinkers.forEach((parts, token) => {
      const { kinds, nodes } = parts;
      nodesByUrl.forEach((node) => {
        if (node.reflection?.kindOf(kinds)) {
          const linkableName = node.name;
          if (nodes.get(linkableName) === void 0) {
            nodes.set(linkableName, node);
          } else {
            console.log(`Duplicate ${token}: ${linkableName}`);
            nodes.set(linkableName, null);
          }
        }
      });
      if (nodes.size > 0) {
        parts.regex = new RegExp(
          `(?<=<span class="${token}">)(` + [...nodes.keys()].join("|") + ")(?=</span>)",
          "gms"
        );
      }
    });
  }
  html = html.replace(
    /<code class="language-(diff-)?(.*?)">(.*?)<\/code>/gms,
    (_pre, diff, lang, code) => {
      if (languages[lang] == null) {
        loadLanguages([lang]);
      }
      code = removeMagicComments(code);
      code = highlight(
        decode(code),
        languages[diff != null ? "diff" : lang],
        (diff != null ? "diff-" : "") + lang
      );
      code = code.replace(
        /<span class="plain-text">\n<span class="prefix">.<\/span><\/span>/gms,
        ""
      );
      codeLinkers.forEach(({ nodes, regex }) => {
        code = regex == null ? code : code.replace(regex, (token) => {
          const linkNode = nodes.get(token);
          if (linkNode == null) {
            return token;
          }
          return `<a href="${isSingle ? "#" : ""}${linkNode.url}">${token}</a>`;
        });
      });
      return "<code>" + code + "</code>";
    }
  );
  html = html.replace(
    /(<section class="s\d"><h\d>Mentions<\/h\d>)(.*?)(<\/section>)/gms,
    ""
  );
  return minify(html, { collapseWhitespace: true });
};

const getIndexOf = (sortList, str, defaultIndex = 0) => {
  const exactIndex = sortList.indexOf(str);
  if (exactIndex > -1) {
    return exactIndex;
  }
  const regexIndex = sortList.findIndex(
    (regex) => regex instanceof RegExp && regex.test(str)
  );
  return regexIndex > -1 ? regexIndex : defaultIndex;
};
const getSorter = (sortList) => {
  const star = getIndexOf(sortList, "*");
  return (str1, str2) => {
    const index1 = getIndexOf(sortList, str1, star);
    const index2 = getIndexOf(sortList, str2, star);
    return index1 > index2 ? 1 : index1 == index2 ? str1 > str2 ? 1 : -1 : -1;
  };
};
const forEachNode = (node, callback) => {
  node.children.forEach((node2) => forEachNode(node2, callback));
  callback(node);
};
const createDocs = (baseUrl, rootDir, reUseDir = false) => {
  const apiFiles = [];
  const markdownDirs = [];
  const jsFiles = [];
  const lessFiles = [];
  const files = [];
  const stringFiles = [];
  const concatFiles = [];
  const rootNode = createNode("root", "Home");
  const nodesByUrl = /* @__PURE__ */ new Map();
  const addComponentForEachNodeIfEqual = (url, component, file = "index.html", isSingle = false, output = "html", orChild = false) => forEachNode(rootNode, (node) => {
    if (node.url === url || orChild && node.url.startsWith(url)) {
      node.pages.set(file, {
        pageComponent: component ?? DefaultPage,
        isSingle,
        output
      });
    }
  });
  const addEntry = (file, suffixes, collection, extra = null) => {
    exists(file) && suffixes.some((suffix) => file.endsWith(suffix)) ? collection.push([file, extra]) : null;
  };
  const addEntries = (dir, suffixes, collection, extra = null) => forEachDeepFile(dir, (file) => addEntry(file, suffixes, collection, extra));
  const docs = {
    addApiDir: (apiDir) => {
      addEntries(apiDir, [".d.ts"], apiFiles);
      return docs;
    },
    addApiFile: (apiFile) => {
      addEntry(apiFile, [".d.ts"], apiFiles);
      return docs;
    },
    addRootMarkdownFile: (rootMarkdownFile) => {
      const { name = "Home", summary, body } = getMarkdown(rootMarkdownFile);
      rootNode.name = name;
      rootNode.summary = summary;
      rootNode.body = body;
      return docs;
    },
    addMarkdownDir: (markdownDir, executable) => {
      addEntry(markdownDir, [""], markdownDirs, executable);
      return docs;
    },
    addJsDir: (dir, destinationDir = "js") => {
      addEntries(dir, [".js", ".ts", ".jsx", ".tsx"], jsFiles, destinationDir);
      return docs;
    },
    addJsFile: (file, destinationDir = "js") => {
      addEntry(file, [".js", ".ts", ".jsx", ".tsx"], jsFiles, destinationDir);
      return docs;
    },
    addLessDir: (dir, destinationDir = "css") => {
      addEntries(dir, [".less", ".css"], lessFiles, destinationDir);
      return docs;
    },
    addLessFile: (file, destinationDir = "css") => {
      addEntry(file, [".less", ".css"], lessFiles, destinationDir);
      return docs;
    },
    addDir: (dir, destinationDir) => {
      addEntries(dir, [""], files, destinationDir ?? "");
      return docs;
    },
    addFile: (file, destinationDir) => {
      addEntry(file, [""], files, destinationDir);
      return docs;
    },
    addFiles: (dir, suffixes, destinationDir) => {
      addEntries(dir, suffixes, files, destinationDir ?? "");
      return docs;
    },
    addStringFile: (content, destinationDir) => {
      stringFiles.push([content, destinationDir]);
      return docs;
    },
    addConcatFiles: (files2, destinationDir) => {
      concatFiles.push([files2, destinationDir]);
      return docs;
    },
    generateNodes: (nodeSorters) => {
      markdownDirs.forEach(
        ([markdownDir, executable]) => createMarkdownNode(markdownDir, rootNode, executable)
      );
      if (apiFiles.length > 0) {
        const apiRoot = createNode("api", "API", rootNode);
        const application = new Application();
        const paths = apiFiles.map((path) => path[0]);
        application.bootstrap({ entryPoints: paths, sort: ["source-order"] });
        application.options.setCompilerOptions(
          paths,
          { strict: true, skipLibCheck: true },
          []
        );
        application.converter.on(
          Converter.EVENT_CREATE_DECLARATION,
          (_, reflection) => {
            reflection.reflectionComment = reflection.comment;
          }
        );
        const project = application.convert();
        if (project != null) {
          mountProjectIntoNode(project, apiRoot, nodeSorters);
        }
      }
      forEachNode(rootNode, (node) => nodesByUrl.set(node.url, node));
      forEachNode(rootNode, prePublish);
      return docs;
    },
    addPageForNode: (url, pageComponent, file, single) => {
      addComponentForEachNodeIfEqual(url, pageComponent, file, single);
      return docs;
    },
    addPageForEachNode: (url, pageComponent, file) => {
      addComponentForEachNodeIfEqual(
        url,
        pageComponent,
        file,
        false,
        "html",
        true
      );
      return docs;
    },
    addTextForNode: (url, textComponent, file, single) => {
      addComponentForEachNodeIfEqual(url, textComponent, file, single, "text");
      return docs;
    },
    addTextForEachNode: (url, textComponent, file) => {
      addComponentForEachNodeIfEqual(
        url,
        textComponent,
        file,
        false,
        "text",
        true
      );
      return docs;
    },
    addMarkdownForNode: (url, markdownComponent, file, single) => {
      addComponentForEachNodeIfEqual(
        url,
        markdownComponent,
        file,
        single,
        "markdown"
      );
      return docs;
    },
    addMarkdownForEachNode: (url, markdownComponent, file) => {
      addComponentForEachNodeIfEqual(
        url,
        markdownComponent,
        file,
        false,
        "markdown",
        true
      );
      return docs;
    },
    publish: () => {
      if (!reUseDir) {
        rmDir(rootDir);
      }
      makeDir(rootDir);
      forEachNode(
        rootNode,
        (pageNode) => pageNode.pages.forEach((page) => {
          const { pageComponent, isSingle } = page;
          page.html = renderToStaticMarkup(
            createElement(PageWithContext, {
              baseUrl,
              rootNode,
              pageNode,
              allNodes: [...nodesByUrl.values()],
              isSingle,
              pageComponent
            })
          );
        })
      );
      forEachNode(
        rootNode,
        (pageNode) => pageNode.pages.forEach((page, file) => {
          const content = decorate(page, nodesByUrl, baseUrl).trim();
          if (content != "") {
            makeFile(join(rootDir, pageNode.url, file), content);
          }
        })
      );
      jsFiles.forEach(
        ([file, destinationDir]) => buildSync({
          entryPoints: [file],
          outdir: makeDir(join(rootDir, destinationDir)),
          minify: true,
          bundle: true
        })
      );
      lessFiles.forEach(
        ([file, destinationDir]) => less.render(readFile(file), {
          filename: file,
          compress: true
        }).then(
          ({ css }) => makeFile(
            join(
              rootDir,
              destinationDir,
              basename(file).replace(/\.less$/, ".css")
            ),
            css
          )
        )
      );
      files.forEach(
        ([file, destinationDir]) => copyFile(file, join(rootDir, destinationDir))
      );
      stringFiles.forEach(
        ([content, destinationFile]) => makeFile(join(rootDir, destinationFile), content)
      );
      concatFiles.forEach(
        ([files2, destinationFile]) => makeFile(
          destinationFile,
          files2.reduce((out, file) => out + readFile(file), "")
        )
      );
    }
  };
  return docs;
};

const NodeChildren = ({ node }) => getSkippedChildren(node).map((node2, key) => /* @__PURE__ */ React.createElement("li", { key }, /* @__PURE__ */ React.createElement(NodeName, { node: node2, link: true })));

export { Markdown, NodeBreadcrumbs, NodeChildren, NodeName, NodeNavigation, NodeSection, NodeSummary, Table, createDocs, forEachDirAndFile, getSkippedChildren, getSorter, isAncestor, isAncestorOrSame, useAllNodes, useBaseUrl, useIsSingle, usePageNode, useRootNode };
